@@ -24,7 +24,7 @@ def get_supervisord_log_file_path():
 
 
 def get_log_file_path():
-    return os.path.join(get_directory_path(), 'uploads.log')
+    return os.path.join(get_directory_path(), 'oort.log')
 
 
 def get_db_file_path():
@@ -92,4 +92,4 @@ def get_config_upload_folder_sections():
         return []
     config = ConfigParser()
     config.read(conf_file_path)
-    return [dict(config[section]) for section in config.sections() if section.startswith('upload-folder-')]
+    return [dict(config[section]) for section in config.sections() if section.startswith('watch-folder-')]
