@@ -53,7 +53,7 @@ class Identity(object):
 
     def save_with_folder(self, upload_folder):
         folder_hash = hashlib.shake_128(upload_folder.encode('utf8')).hexdigest(3)
-        write_config_section_values(f'upload-folder-{folder_hash}',
+        write_config_section_values(f'watch-folder-{folder_hash}',
                                     username=ArcsecondAPI.username(),
                                     api_key=ArcsecondAPI.api_key(debug=self.debug),
                                     organisation=self.organisation or '',
