@@ -2,11 +2,19 @@ import os
 from typing import Optional, Type
 
 from arcsecond import ArcsecondAPI
-from peewee import DoesNotExist
-
 from oort.shared.config import get_logger
 from oort.shared.identity import Identity
-from oort.shared.models import BaseModel, Dataset, NightLog, Organisation, Telescope
+from oort.shared.models import (
+    BaseModel,
+    Dataset,
+    NightLog,
+    Organisation,
+    Status,
+    Substatus,
+    Telescope
+)
+from peewee import DoesNotExist
+
 from .errors import UploadPreparationAPIError, UploadPreparationError, UploadPreparationFatalError
 from .packer import UploadPack
 
@@ -23,6 +31,7 @@ class UploadPreparator(object):
         self._preparation_succeeded = False
         self._preparation_can_be_restarted = False
 
+        self._organisation = None
         self._telescope = None
         self._night_log = None
         self._obs_or_calib = None
@@ -30,9 +39,11 @@ class UploadPreparator(object):
 
         # Do NOT mix debug and self._identity.debug
 
+        self._pack.upload.smart_update(astronomer=self._identity.username)
         if self._identity.organisation:
             api = ArcsecondAPI.organisations(debug=self._identity.debug)
-            self._sync_local_resource(Organisation, api, subdomain=self._identity.organisation)
+            self._organisation = self._sync_local_resource(Organisation, api, self._identity.organisation)
+            self._pack.upload.smart_update(organisation=self._organisation)
 
     # ------ PROPERTIES ------------------------------------------------------------------------------------------------
 
@@ -79,15 +90,15 @@ class UploadPreparator(object):
 
     # ------ SYNC ------------------------------------------------------------------------------------------------------
 
-    def _sync_local_resource(self, db_class: Type[BaseModel], api: ArcsecondAPI, **kwargs) -> BaseModel:
-        remote_resource, error = api.read(kwargs)
+    def _sync_local_resource(self, db_class: Type[BaseModel], api: ArcsecondAPI, id_value) -> BaseModel:
+        remote_resource, error = api.read(id_value)
         if error:
             raise UploadPreparationAPIError(str(error))
 
         try:
-            resource = db_class.smart_get(**kwargs)
+            resource = db_class.smart_get(**{db_class._primary_field: id_value})
         except DoesNotExist:
-            resource = self._create_local_resource(db_class, **kwargs)
+            resource = self._create_local_resource(db_class, **{db_class._primary_field: id_value})
 
         return resource
 
@@ -98,13 +109,18 @@ class UploadPreparator(object):
             resource = db_class.smart_get(**kwargs)
 
         except DoesNotExist:
-            self._logger.info(f'{self.prefix} Local resource does not exists. Find or create remote one.')
+            self._logger.info(f'{self.prefix} Local resource does not exist. Find or create remote one.')
 
             remote_resource = self._find_or_create_remote_resource(api, **kwargs)
             if remote_resource is None:
                 raise UploadPreparationError('cant create resource')
 
-            self._logger.info(f'{self.prefix} Remote resource ok.')
+            common_keys = kwargs.keys() & remote_resource.keys()
+            common_values = [(k, kwargs.get(k), remote_resource.get(k)) for k in common_keys]
+            if any([(k, v1, v2) for k, v1, v2 in common_values if v1 != v2]):
+                msg = f'Mismatch between remote and local for {db_class} resource: {common_values}'
+                raise UploadPreparationError(msg)
+
             resource = self._create_local_resource(db_class, **remote_resource)
 
         else:
@@ -132,7 +148,7 @@ class UploadPreparator(object):
         # The resource exists. Do nothing.
         elif len(response_list) == 1:
             new_resource = response_list[0]
-            # new_resource = self._check_resource_name(api, response_list[0], kwargs_name)
+            self._logger.info(f'{self.prefix} Remote resource exists, using it.')
 
         # Multiple resources found ??? Filter is not good, or something fishy is happening.
         else:
@@ -149,22 +165,26 @@ class UploadPreparator(object):
         remote_resource, error = api.create(kwargs)
 
         if error is not None:
-            msg = f'Failed to create resource in {api} endpoint. Retry is automatic.'
+            msg = f'Failed to create resource in {api} endpoint: {str(error)}'
             raise UploadPreparationError(msg)
         else:
+            self._logger.info(f'{self.prefix} Remote resource created.')
             return remote_resource
 
     # ------------------------------------------------------------------------------------------------------------------
 
     def _create_local_resource(self, db_class: Type[BaseModel], **kwargs):
-        self._logger.info('Creating local resource.')
+        self._logger.info(f'{self.prefix} Creating local resource.')
 
         fields = {k: v for k, v in kwargs.items() if k in db_class._meta.sorted_field_names and v is not None}
 
-        if self._identity.organisation and 'organisation' in db_class._meta.get_field_names():
+        if self._identity.organisation and 'organisation' in db_class._meta.sorted_field_names:
             fields.update(organisation=self._identity.organisation)
 
-        return db_class.smart_create(**fields)
+        instance = db_class.smart_create(**fields)
+        self._logger.info(f'{self.prefix} Local resource created.')
+
+        return instance
 
     # ------ CHECKS ----------------------------------------------------------------------------------------------------
 
@@ -174,12 +194,12 @@ class UploadPreparator(object):
 
         self._logger.info(f'{self.prefix} Reading telescope {self._identity.telescope}...')
         api = ArcsecondAPI.telescopes(**self.api_kwargs)
-        self._telescope = self._sync_local_resource(Telescope, api, uuid=self._identity.telescope)
+        self._telescope = self._sync_local_resource(Telescope, api, self._identity.telescope)
 
     # ------------------------------------------------------------------------------------------------------------------
 
     def _sync_night_log(self):
-        self._logger.info(f'{self.prefix} Syncing nightlog {self._pack.night_log_date_string}...')
+        self._logger.info(f'{self.prefix} Syncing night log {self._pack.night_log_date_string}...')
 
         kwargs = {'date': self._pack.night_log_date_string}
         if self._identity.telescope:
@@ -191,9 +211,11 @@ class UploadPreparator(object):
     # ------------------------------------------------------------------------------------------------------------------
 
     def _sync_observation_or_calibration(self):
-        self._logger.info(f'{self.prefix} Syncing {self._pack.remote_resources_name}...')
         resources_api = getattr(ArcsecondAPI, self._pack.remote_resources_name)(**self.api_kwargs)
         kwargs = {'night_log': str(self._night_log.uuid), 'name': self._pack.dataset_name}
+        if self._pack.resource_type == 'observation':
+            kwargs.update(target_name=self._pack.dataset_name)
+        self._logger.info(f'{self.prefix} Syncing {self._pack.remote_resources_name}: {kwargs}...')
         self._obs_or_calib = self._sync_resource(self._pack.resource_db_class, resources_api, **kwargs)
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -206,27 +228,40 @@ class UploadPreparator(object):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    async def prepare(self):
+    def prepare(self):
         self._logger.info(f'Preparation started for {self._pack.file_path}')
         try:
-            self._pack.save(status='Preparing', substatus='Syncing Telescope...')
+            self._pack.upload.smart_update(status=Status.PREPARING.value, substatus=Substatus.SYNC_TELESCOPE.value)
             self._sync_telescope()
-            self._pack.save(status='Preparing', substatus='Syncing Night Log...')
+
+            if self._telescope:
+                self._pack.upload.smart_update(telescope=self._telescope)
+
+            self._pack.upload.smart_update(substatus=Substatus.SYNC_NIGHTLOG.value)
             self._sync_night_log()
-            self._pack.save(status='Preparing', substatus='Syncing Observation/Calibration...')
+
+            self._pack.upload.smart_update(substatus=Substatus.SYNC_OBS_OR_CALIB.value)
             self._sync_observation_or_calibration()  # observation or calibration
-            self._pack.save(status='Preparing', substatus='Syncing Dataset...')
+
+            self._pack.upload.smart_update(substatus=Substatus.SYNC_DATASET.value)
             self._sync_dataset()
-            self._pack.save(dataset=self.dataset)
+
+            if self._dataset:
+                self._pack.upload.smart_update(dataset=self.dataset)
+
         except UploadPreparationFatalError as e:
             self._logger.info(f'Preparation failed for {self._pack.file_path} with error: {str(e)}')
+            self._pack.upload.smart_update(status=Status.ERROR.value, substatus=Substatus.ERROR.value, error=str(e))
             self._preparation_succeeded = False
             self._preparation_can_be_restarted = False
+
         except UploadPreparationError as e:
             self._logger.info(f'Preparation failed for {self._pack.file_path} with error: {str(e)}')
+            self._pack.upload.smart_update(status=Status.ERROR.value, substatus=Substatus.ERROR.value, error=str(e))
             self._preparation_succeeded = False
             self._preparation_can_be_restarted = True
+
         else:
             self._logger.info(f'Preparation succeeded for {self._pack.file_path}')
-            self._pack.save(status='Ready', substatus='')
+            self._pack.upload.smart_update(status=Status.UPLOADING.value, substatus=Substatus.READY.value)
             self._preparation_succeeded = True
